<?php

namespace Lc\CaracoleBundle\Repository\Product;

use Knp\Component\Pager\PaginatorInterface;
use Lc\CaracoleBundle\Repository\SectionRepositoryQueryTrait;
use Lc\CaracoleBundle\Repository\StatusRepositoryQueryTrait;
use Lc\CaracoleBundle\Repository\TreeRepositoryQueryTrait;
use Lc\SovBundle\Repository\AbstractRepositoryQuery;

class ProductCategoryRepositoryQuery extends AbstractRepositoryQuery
{
    use SectionRepositoryQueryTrait;
    use StatusRepositoryQueryTrait;
    use TreeRepositoryQueryTrait;

    protected $isJoinProductFamilies = false;


    public function __construct(ProductCategoryRepository $repository, PaginatorInterface $paginator)
    {
        parent::__construct($repository, 'r', $paginator);
    }

    public function joinProductFamilies(): self
    {
        if (!$this->isJoinProductFamilies) {
            $this->isJoinProductFamilies = true;

            return $this
                    ->innerJoin('.productFamilies', 'pf');
        }

        return $this;
    }
    public function hasProductFamilyOnline(): self
    {
        $this->joinProductFamilies();

        return $this->andWhere('pf.status = 1');
    }

    public function filterByCurrentDay(): self
    {
        return $this->andWhere('e.displaySpecificDay IS NULL OR e.displaySpecificDay = :dayToday')
                ->setParameter('dayToday', date('N'));
    }
}
