<?php

namespace Lc\CaracoleBundle\Repository\Product;

use Lc\CaracoleBundle\Model\Product\ProductCategoryInterface;
use Lc\CaracoleBundle\Repository\SectionStoreTrait;
use Lc\SovBundle\Repository\AbstractStore;

class ProductCategoryStore extends AbstractStore
{
    use SectionStoreTrait;

    protected ProductCategoryRepositoryQuery $query;

    public function __construct(ProductCategoryRepositoryQuery $query)
    {
        $this->query = $query;
    }

    public function getParents()
    {
        $query = $this->query->create();
        if ($this->section) {
            $query->filterBySection($this->section);
        }
        $query->filterIsParent();

        return $query->find();
    }

    //findAllByParent
    public function getByParent(
            ProductCategoryInterface $parentCategory,
            bool $withOffline = false,
            bool $withEmptyCategories = true,
            bool $filterBySpecificDay = false
    ): array {

        $query = $this->query->create();
        $query->filterByParent($parentCategory);

        if ($withOffline) {
            $query->filterIsOnlineAndOffline();
        } else {
            $query->filterIsOnline();
        }

        if (!$withEmptyCategories) {
            $query->hasProductFamilyOnline();
        }

        if ($filterBySpecificDay) {
            $query->filterBySpecificDay();
        }

        $query->orderBy('position', 'asc');

        return $query->find();
    }

}
